/* main.c - Entry point - Don Yang / Dave Newquist / John Rapp

   07/26/01
                                                                           */

#include<signal.h>
#include<stdio.h>
#include<stdlib.h>
#include<time.h>
#include<unistd.h>
#include"main.h"
#include"opt.h"
#include"token.h"
#include"tree.h"


/* Exported globals */
int LimitBreak;

/* Local functions */
static void AlarmHandler(int a);

/******************************************************************** main */
int main(int argc, char **argv)
{
   unsigned int timelimit;
   time_t t;
   Input *input;
   Node *tree;

   /* Load input */
   t = time(NULL);
   if( (input = InitInput(stdin)) == NULL )        /* (token.c) */
      return 1;
   tree = CreateParseTree(input);                  /* (tree.c) */
   DeleteInput(input);                             /* (token.c) */

   /* Estimate amount of panic time */
   t = (time(NULL) - t) * 2 + 2;

   /* Set alarm */
   timelimit = (unsigned int)((argc > 1) ? atoi(argv[1]) : 180);
   timelimit -= (unsigned int)t;
   if( timelimit < 0 )
      timelimit = 5;
   LimitBreak = 0;
   signal(SIGALRM, AlarmHandler);

   alarm(timelimit);


   /* Optimize parse tree */
   Optimize(&tree);                                /* (opt.c) */


   /* Write output */
   DumpTree(tree, stdout);                         /* (tree.c) */
   DeleteParseTree(tree);                          /* (tree.c) */
   signal(SIGALRM, SIG_DFL);
   return 0;

} /* main() */


/************************************************************ AlarmHandler */
static void AlarmHandler(int a)
{
   #ifndef NDEBUG
      fputs("SIGALRM received\n", stderr);
   #endif
   LimitBreak = 1;

} /* AlarmHandler() */


#ifndef NDEBUG

/*************************************************************** DebugNode */
void DebugNode(Node *node)
{
   fputc(node->attr.b  ? 'B' : '-', stderr);
   fputc(node->attr.em ? 'E' : '-', stderr);
   fputc(node->attr.i  ? 'I' : '-', stderr);
   fputc(node->attr.s  ? 'S' : '-', stderr);
   fputc(node->attr.tt ? 'T' : '-', stderr);
   fprintf(stderr, "-u%ds%cc%c ", node->attr.u,
      node->attr.size > 0 ? node->attr.size : '-',
      node->attr.color > 0 ? node->attr.color : '-');
   DebugToken(node->token);

} /* DebugNode() */

/************************************************************** DebugToken */
void DebugToken(Token *token)
{
   switch( token->type )
   {
      case TokenEOF:
         fputs("eof\n", stderr);
         break;
      case TokenText:
         fprintf(stderr, "text: [%s]\n", token->lexeme.s);
         break;
      case TokenSpace:
         fprintf(stderr, "space: [%02x]\n", token->lexeme.i);
         break;
      case TagB:
         fputs("B\n", stderr);
         break;
      case TagEM:
         fputs("EM\n", stderr);
         break;
      case TagI:
         fputs("I\n", stderr);
         break;
      case TagPL:
         fputs("PL\n", stderr);
         break;
      case TagS:
         fputs("S\n", stderr);
         break;
      case TagTT:
         fputs("TT\n", stderr);
         break;
      case TagU:
         fputs("U\n", stderr);
         break;
      case TagSize:
         fprintf(stderr, "size: %c\n", token->lexeme.i);
         break;
      case TagColor:
         fprintf(stderr, "color: %c\n", token->lexeme.i);
         break;
      default:
         fputs("Error!\n", stderr);
         break;
   }

} /* DebugToken() */

/*************************************************************** DebugTree */
void DebugTree(Node *node, int level)
{
   int i;

   while( node )
   {
      for(i = 0; i < level; i++)
         fputs("   ", stderr);
      DebugNode(node);
      if( node->tree )
         DebugTree(node->tree, level + 1);
      node = node->next;
   }

} /* DebugTree() */

#endif
