/* o_dup.c - Generic redundancy - Don Yang / Dave Newquist / John Rapp

   Need to call o_empty to remove empty ranges before calling o_duplicate.

   07/26/01
                                                                           */

#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include"smlng.h"
#include"token.h"
#include"tree.h"


/* Local functions */
static void MergeNeighbors(Node *node);
static int RedundantAttribute(Node *node);
static int SameTag(Token *a, Token *b);

/************************************************************* o_duplicate */
int o_duplicate(Node **node)
{
   int modified;

   modified = 0;
   while( *node )
   {
      if( (*node)->token->tagtype )
      {
         /* Tags */
         if( (*node)->next != NULL &&
             SameTag((*node)->token, (*node)->next->token) )
         {
            MergeNeighbors(*node);
            modified = 1;
         }
         else if( RedundantAttribute(*node) )
         {
            PromoteSubtree(node);            /* (tree.c) */
            modified = 1;
         }
         else
         {
            modified |= o_duplicate(&((*node)->tree));
            node = &((*node)->next);
         }
      }
      else
      {
         /* Text */
         node = &((*node)->next);
      }
   }

   return modified;

} /* o_duplicate() */


/********************************************************** MergeNeighbors */
static void MergeNeighbors(Node *node)
{
   Node *tail, *del;

   del = node->next;

   for(tail = node->tree; tail->next; tail = tail->next);
   tail->next = del->tree;

   node->next = del->next;
   del->next = del->tree = NULL;
   DeleteParseTree(del);

} /* MergeNeighbors() */

/****************************************************** RedundantAttribute */
static int RedundantAttribute(Node *node)
{
   switch( node->token->type )
   {
      case TagB:     return node->attr.b;
      case TagI:     return node->attr.i;
      case TagS:     return node->attr.s;
      case TagTT:    return node->attr.tt;

      case TagEM:    return node->attr.s;

      case TagSize:  return node->attr.size == node->token->lexeme.i;
      case TagColor: return node->attr.color == node->token->lexeme.i;
      default:
         return 0;
   }

} /* RedundantAttribute() */

/***************************************************************** SameTag */
static int SameTag(Token *a, Token *b)
{
   if( a->type != b->type )
      return 0;
   if( a->type == TagSize || a->type == TagClose )
      return a->lexeme.i == b->lexeme.i;
   return 1;

} /* SameTag() */
