/* tree.c - Create parse tree - Don Yang / Dave Newquist / John Rapp

   07/26/01
                                                                           */

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include"smlng.h"
#include"token.h"
#include"tree.h"


/* Local functions */
static Node *CreateParseTreeR(Input *input, Token *tag, Attr attr);

/********************************************************* CreateParseTree */
Node *CreateParseTree(Input *input)
{
   Attr attr;

   SetDefaultAttribute(&attr);         /* (smlng.c) */
   return CreateParseTreeR(input, NULL, attr);

} /* CreateParseTree() */

/************************************************************** DeleteNode */
void DeleteNode(Node **node)
{
   Node *del;

   del = *node;
   *node = del->next;
   del->next = NULL;
   DeleteParseTree(del);

} /* DeleteNode() */

/********************************************************* DeleteParseTree */
void DeleteParseTree(Node *root)
{
   Node *next;

   while( root != NULL )
   {
      if( root->tree )
         DeleteParseTree(root->tree);

      next = root->next;
      DeleteToken(root->token);     /* (token.c) */
      free(root);
      root = next;
   }

} /* DeleteParseTree() */

/**************************************************************** DumpTree */
void DumpTree(Node *node, FILE *output)
{
   while( node != NULL )
   {
      switch( node->token->type )
      {
         case TokenText:
            fputs(node->token->lexeme.s, output);
            break;
         case TokenSpace:
            fputc(node->token->lexeme.i, output);
            break;

         case TagB:
            fputs("<B>", output);   DumpTree(node->tree, output);
            fputs("</B>", output);
            break;
         case TagEM:
            fputs("<EM>", output);  DumpTree(node->tree, output);
            fputs("</EM>", output);
            break;
         case TagI:
            fputs("<I>", output);   DumpTree(node->tree, output);
            fputs("</I>", output);
            break;
         case TagPL:
            fputs("<PL>", output);  DumpTree(node->tree, output);
            fputs("</PL>", output);
            break;
         case TagS:
            fputs("<S>", output);   DumpTree(node->tree, output);
            fputs("</S>", output);
            break;
         case TagTT:
            fputs("<TT>", output);  DumpTree(node->tree, output);
            fputs("</TT>", output);
            break;
         case TagU:
            fputs("<U>", output);   DumpTree(node->tree, output);
            fputs("</U>", output);
            break;

         default: /* TagSize / TagColor */
            fprintf(output, "<%c>", node->token->lexeme.i);
            DumpTree(node->tree, output);
            fprintf(output, "</%c>", node->token->lexeme.i);
            break;
      }
      node = node->next;
   }

} /* DumpTree() */

/********************************************************** PromoteSubtree */
void PromoteSubtree(Node **node)
{
   Node *del, *tail;

   del = *node;
   if( del->tree == NULL )
   {
      *node = del->next;
      del->next = NULL;
      DeleteParseTree(del);
      return;
   }

   for(tail = del->tree; tail->next != NULL; tail = tail->next);
   *node = del->tree;
   tail->next = del->next;

   del->next = del->tree = NULL;
   DeleteParseTree(del);

} /* PromoteSubtree() */


/******************************************************** CreateParseTreeR */
static Node *CreateParseTreeR(Input *input, Token *tag, Attr attr)
{
   Node *root, *node, *prev;
   Token *token;

   /* Set attribute for current range */
   SetAttribute(&attr, tag);

   /* Parse tokens */
   root = prev = NULL;
   while( (token = GetToken(input)) != NULL )      /* (token.c) */
   {
      /* Check token */
      if( token->type == TokenEOF || token->tagtype == TagClose )
      {
         /* End of file / subtree */
         DeleteToken(token);                       /* (token.c) */
         return root;
      }

      /* Create node */
      if( (node = (Node*)malloc(sizeof(Node))) == NULL )
      {
         fputs("Out of memory\n", stderr);
         DeleteToken(token);
         return NULL;
      }

      /* Link sibling */
      if( root == NULL )
         root = node;
      else
         prev->next = node;

      node->token = token;
      node->next = node->tree = NULL;
      memcpy(&(node->attr), &attr, sizeof(Attr));
      if( token->tagtype == TagOpen )
      {
         /* Parse subtree */
         node->tree = CreateParseTreeR(input, node->token, attr);
      }
      prev = node;
   }

   /* Error */
   DeleteParseTree(root);
   return NULL;

} /* CreateParseTreeR() */
